/** @file OTClient.h
*   @brief OTClient class definition
*
*   Copyright (C) 2004,2005 Opentick corporation. All rights reserved.
*/

#ifndef _OT_CLIENT_H
#define _OT_CLIENT_H

#include <vector>

#include "OTSocket.h"
#include "Threads.h"
#include "OTCommands.h"
#include "OTResponse.h"
#include "OTDataEntity.h"
#include "OTQuote.h"
#include "OTMMQuote.h"
#include "OTTrade.h"
#include "OTError.h"
#include "OTMessage.h"
#include "OTExchange.h"
#include "OTSymbol.h"
#include "OTOHLC.h"
#include "OTConst.h"  
#include "OTDef.h"
#include "OTEquityInit.h"
#include "OTBookCancel.h"
#include "OTBookChange.h"
#include "OTBookDelete.h"
#include "OTBookExecute.h"
#include "OTBookOrder.h"
#include "OTBookPriceLevel.h"
#include "OTBookPurge.h"
#include "OTBookReplace.h"
#include "OTBBO.h"
#include "OTSplit.h"
#include "OTDividend.h"
#include "OTOptionInit.h"
  

  typedef struct
  {
    int           reqId; 
    OTDataEntity  *data;
  } RequestsVectorElement_t;

  using namespace std;

  /**  This is the main OTFeed class.  */
  class   OTClient 
  {
    public:
      /**  Default constructor. */
      OTClient ();

      /**  Destructor.   */
      virtual ~OTClient ();

      /** 
       *  Requests a new real-time tick stream for the symbol specified 
       *  by the OTDataEntity object. Incoming market events appropriate 
       *  to the requested data entity will be passed to the application.
       *  This request allows you to limit the type of incoming messages
       *  you will receive by using the mask parameter.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param mask  Represents events that you need on specified symbol
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestTickStream (&OTDataEntity("Q","DELL"), OT_TICK_TYPE_LEVEL1);   
       *  @endcode 
       */
      int requestTickStream (OTDataEntity *dataEntity, int mask = OT_TICK_TYPE_ALL, int snapshotMask = 0);
   
      /** 
       *  This is a convenience function which will request level II data. 
       *  It will transparently request book events or MMQuotes, depending on the symbol and
       *  exchange specified in the OTDataEntity object. This function exists because not all 
       *  exchanges send book events, and not all exchanges send market maker quotes.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestMarketDepth (&OTDataEntity("Q","YHOO"));   
       *  @endcode 
       */
      int requestMarketDepth(OTDataEntity *dataEntity);

      /**  
       *  Requests a recent ticks for the symbol specified by the OTDataEntity object.
       *  Incoming market events appropriate to the requested data entity will be passed
       *  to the application. This request allows you to limit the type of incoming messages
       *  you will receive by using the snapshotMask parameter.
       *  Note that level II data events are not provided on this request.
       *  
       *  @param entity  Represents entity which data you need
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestTickSnapshot (&OTDataEntity("N","GM"), OT_TICK_TYPE_TRADE);   
       *  @endcode 
       */
      int requestTickSnapshot (OTDataEntity *entity, int snapshotMask = OT_TICK_TYPE_ALL);
      
      /** 
       *  Equity Initialize data for the current day for the symbol specified by the OTDataEntity object.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestEquityInit(&OTDataEntity("Q", "GOOG"));  
       *  @endcode 
       */
      int requestEquityInit (OTDataEntity *dataEntity);

      /** 
       *  Requests a new real-time book stream, specified by the OTDataEntity parameter.
       *  Incoming book events appropriate to the requested data entity will be passed to the application.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param mask  Represents events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestBookStream(&OTDataEntity("is", "MSFT"), OT_BOOK_TYPE_EXECUTE);  
       *  @endcode 
       */
      int requestBookStream (OTDataEntity *dataEntity, int mask = OT_BOOK_TYPE_ALL);

      /** 
       *  Requests a new real-time option chain specified by the OTDataEntity and a variation
       *  of optional arguments including expiry date and minimum and maximum strike price defintions. 
       *  Incoming market events appropriate to the request will be passed to the application. 
       *  This request additionally allows you to limit the type of incoming messages you will
       *  recieve by using the mask parameter
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param mask  Represents events that you need on specified symbol
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChain(&OTDataEntity("AO", "MSFT"), OT_TICK_TYPE_TRADE, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChain(OTDataEntity *dataEntity, int mask = OT_TICK_TYPE_ALL, int snapshotMask = 0);

      /** 
       *  Requests a new real-time option chain specified by the OTDataEntity and a variation
       *  of optional arguments including expiry date and minimum and maximum strike price defintions. 
       *  Incoming market events appropriate to the request will be passed to the application. 
       *  This request additionally allows you to limit the type of incoming messages you will
       *  recieve by using the mask parameter
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param mask  Represents events that you need on specified symbol
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChain(&OTDataEntity("AO", "MSFT"), 10, 45, OT_TICK_TYPE_TRADE, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChain(OTDataEntity *dataEntity, double minStrikePrice, double maxStrikePrice, int mask = OT_TICK_TYPE_ALL, int snapshotMask = 0);

      /** 
       *  Requests a new real-time option chain specified by the OTDataEntity and a variation
       *  of optional arguments including expiry date and minimum and maximum strike price defintions. 
       *  Incoming market events appropriate to the request will be passed to the application. 
       *  This request additionally allows you to limit the type of incoming messages you will
       *  recieve by using the mask parameter
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @param mask  Represents events that you need on specified symbol
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChain(&OTDataEntity("AO", "MSFT"), 2007, 04, OT_TICK_TYPE_TRADE, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChain(OTDataEntity *dataEntity, int expYear, short int expMonth, int mask = OT_TICK_TYPE_ALL, int snapshotMask = 0);

      /** 
       *  Requests a new real-time option chain specified by the OTDataEntity and a variation
       *  of optional arguments including expiry date and minimum and maximum strike price defintions. 
       *  Incoming market events appropriate to the request will be passed to the application. 
       *  This request additionally allows you to limit the type of incoming messages you will
       *  recieve by using the mask parameter
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @param mask  Represents events that you need on specified symbol
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChain(&OTDataEntity("AO", "MSFT"), 2007, 04, 25, 35, OT_TICK_TYPE_TRADE, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChain(OTDataEntity *dataEntity, int expYear, short int expMonth, double minStrikePrice, double maxStrikePrice, int mask = OT_TICK_TYPE_ALL, int snapshotMask = 0);

      /** 
       *  Requests a recent option chain events specified by the OTDataEntity and
       *  a variation of optional arguments including expiry date and minimum and
       *  maximum strike price definitions. Incoming market events appropriate to
       *  the request will be passed to the application. This request additionally
       *  allows you to limit the type of incoming messages you will receive by using
       *  the snapshotMask parameter (see tick type constants).
       *  Note that level II data events are not provided on this request.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChainSnapshot(&OTDataEntity("AO", "MSFT"), OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChainSnapshot(OTDataEntity *dataEntity, int snapshotMask = OT_TICK_TYPE_ALL);

      /** 
       *  Requests a recent option chain events specified by the OTDataEntity and
       *  a variation of optional arguments including expiry date and minimum and
       *  maximum strike price definitions. Incoming market events appropriate to
       *  the request will be passed to the application. This request additionally
       *  allows you to limit the type of incoming messages you will receive by using
       *  the snapshotMask parameter (see tick type constants).
       *  Note that level II data events are not provided on this request.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChainSnapshot(&OTDataEntity("AO", "MSFT"), 7, 27, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChainSnapshot(OTDataEntity *dataEntity, double minStrikePrice, double maxStrikePrice, int snapshotMask = OT_TICK_TYPE_ALL);

      /** 
       *  Requests a recent option chain events specified by the OTDataEntity and
       *  a variation of optional arguments including expiry date and minimum and
       *  maximum strike price definitions. Incoming market events appropriate to
       *  the request will be passed to the application. This request additionally
       *  allows you to limit the type of incoming messages you will receive by using
       *  the snapshotMask parameter (see tick type constants).
       *  Note that level II data events are not provided on this request.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChainSnapshot(&OTDataEntity("AO", "MSFT"), 2007, 5, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChainSnapshot(OTDataEntity *dataEntity, int expYear, short int expMonth, int snapshotMask = OT_TICK_TYPE_ALL);

      /** 
       *  Requests a recent option chain events specified by the OTDataEntity and
       *  a variation of optional arguments including expiry date and minimum and
       *  maximum strike price definitions. Incoming market events appropriate to
       *  the request will be passed to the application. This request additionally
       *  allows you to limit the type of incoming messages you will receive by using
       *  the snapshotMask parameter (see tick type constants).
       *  Note that level II data events are not provided on this request.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @param snapshotMask  Represents recent events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionChainSnapshot(&OTDataEntity("AO", "MSFT"), 2007, 5, 7, 27, OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestOptionChainSnapshot(OTDataEntity *dataEntity, int expYear, short int expMonth, double minStrikePrice, double maxStrikePrice, int snapshotMask = OT_TICK_TYPE_ALL);

      /** 
       *  Requests Option Initialize events appropriate to the specified OTDataEntity object.
       *  You will receive one option initialize event for each option present for the
       *  underlying symbol specified in the OTDataEntity object.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionInit(&OTDataEntity("AO", "MSFT"));  
       *  @endcode 
       */
      int requestOptionInit(OTDataEntity *dataEntity);

      /** 
       *  Requests Option Initialize events appropriate to the specified OTDataEntity object.
       *  You will receive one option initialize event for each option present for the
       *  underlying symbol specified in the OTDataEntity object.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionInit(&OTDataEntity("AO", "MSFT"), 7.0, 27.0);  
       *  @endcode 
       */
      int requestOptionInit(OTDataEntity *dataEntity, double minStrikePrice, double maxStrikePrice);

      /** 
       *  Requests Option Initialize events appropriate to the specified OTDataEntity object.
       *  You will receive one option initialize event for each option present for the
       *  underlying symbol specified in the OTDataEntity object.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionInit(&OTDataEntity("AO", "MSFT"), 2007, 5);  
       *  @endcode 
       */
      int requestOptionInit(OTDataEntity *dataEntity, int expYear, short int expMonth);

      /** 
       *  Requests Option Initialize events appropriate to the specified OTDataEntity object.
       *  You will receive one option initialize event for each option present for the
       *  underlying symbol specified in the OTDataEntity object.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param expYear  Expiration year of the option, only options with this year would be added to chain
       *  @param expMonth  Expiration month of the option, only options with this month would be added to chain
       *  @param minStrikePrice  Option would be added if its price greater than minStrikePrice
       *  @param maxStrikePrice  Option would be added if its price less than maxStrikePrice
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestOptionInit(&OTDataEntity("AO", "MSFT"), 2007, 5, 7.0, 27.0);  
       *  @endcode 
       */
      int requestOptionInit(OTDataEntity *dataEntity, int expYear, short int expMonth, double minStrikePrice, double maxStrikePrice);

      /** 
       *  Requests a new historical data stream for the specified OTDataEntity 
       *  and start and end dates. These dates are UTC timestamps. 
       *  You may also want to refer to our notes about date and time issues.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param startingDate  Represents beginning of the requested period
       *  @param endingDate  Represents ending of the requested period
       *  @param dataType  Represents kind of requested data
       *  @param intervalValue  Represents interval for OHLC data types, ignored for ticks
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code 
       *  int reqId = requestHistData(&OTDataEntity("Q", "MSFT"), 1162389600, 1162476000, OT_HIST_RAW_TICKS, 1); \n
       *  //or \n
       *  int reqId = requestHistData(&OTDataEntity("Q", "MSFT"), 1162389600, 1162476000, OT_HIST_OHLC_MINUTELY, 5);
       *   
       *  @endcode 
       */
      int requestHistData (OTDataEntity *dataEntity, 
                           int startingDate, 
                           int endingDate, 
                           int dataType, 
                           int intervalValue = 1);
      /** 
       *  Requests a new historical data stream for the symbol specified by the
       *  OTDataEntity object. This function sends a request for OT_HIST_RAW_TICKS.
       *  Additionally, it allows for the limitation of incoming message types as defined by the mask.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param startingDate  Represents beginning of the requested period
       *  @param endingDate  Represents ending of the requested period
       *  @param mask  Represents events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling         
       *  
       *  @code int reqId = requestHistTicks(&OTDataEntity("Q", "MSFT"), 1111111111, 222222222, OT_TICK_TYPE_QUOTE | OT_TICK_TYPE_TRADE);  
       *  @endcode 
       */
      int requestHistTicks (OTDataEntity *dataEntity, 
                           int startingDate, 
                           int endingDate, 
                           int mask);

      /** 
       *  Requests historical book data for the exchange and symbol specified 
       *  by the dataEntity parameter for the time period specified. The startingDate
       *  and endingDate parameters are UTC timestamps.
       *  
       *  @param dataEntity  Represents entity which data you need
       *  @param startingDate  Represents beginning of the requested period
       *  @param endingDate  Represents ending of the requested period
       *  @param mask  Represents events that you need on specified symbol
       *  @return Id of the request that could be used in event handlers and/or while canceling         
       *  
       *  @code int reqId = requestHistBooks(&OTDataEntity("is", "MSFT"), 1162389600, 1162476000); 
       *  @endcode 
       */
      int requestHistBooks (OTDataEntity *dataEntity, 
                           int startingDate, 
                           int endingDate, 
                           int mask);

      /** 
       *  Requests a list of all exchanges a subscriber is entitled to.
       *  
       *  @return Id of the request that could be used in event handlers and/or while canceling         
       */
      int requestListExchanges ();

      /** 
       *  Requests a list of all the symbols available on the specified exchange code.
       *  Note that onListSymbols function is called when part of the symbols list was 
       *  received. After last part of the symbols list is received onMessage would be 
       *  called (end of data message).
       *  
       *  @param exchange  Represents exchange acronym, see www.opentick.com
       *  @param symbol  Represents symbol code or its part
       *  @param options  Represents type of events and type of symbol search
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestListSymbols("Q", "MS", OT_LIST_SYMBOLS_CONTAINS); 
       *  @endcode 
       */
      int requestListSymbols (const char *exchange, const char* symbol = NULL, int options = OT_LIST_SYMBOLS_ALL_TYPES);

      /** 
       *  Requests Open, High, and Low values for the current day for the symbol 
       *  specified by the OTDataEntity object.
       *  
       *  @param entity  Represents entity which data you need
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestTodaysOHL(&OTDataEntity("N", "KO")); 
       *  @endcode 
       */
      int requestTodaysOHL (OTDataEntity *entity);
      
      /** 
       *  Use this method when you need to get splits of the dataEntity.
       *  
       *  @param entity  Represents entity which data you need
       *  @param startingDate  Represents beginning of the requested period
       *  @param endingDate  Represents ending of the requested period
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestSplits(&OTDataEntity("Q", "MSFT"), 0, 222222222);  
       *  @endcode 
       */
      int requestSplits (OTDataEntity *entity, int startingDate, int endingDate);

      /** 
       *  Use this method when you need to get dividends of the dataEntity.
       *  
       *  @param entity  Represents entity which data you need
       *  @param startingDate  Represents beginning of the requested period
       *  @param endingDate  Represents ending of the requested period
       *  @return Id of the request that could be used in event handlers and/or while canceling 
       *  
       *  @code int requestId = requestDividends(&OTDataEntity("Q", "MSFT"), 0, 222222222);  
       *  @endcode 
       */
      int requestDividends (OTDataEntity *entity, int startingDate, int endingDate);
       
      /** 
       *  Use this method when you need not tick stream requested by requestMarketDepth method.
       *  
       *  @param cancelId  Represents request id returned by requestMarketDepth
       */
      void cancelMarketDepth(int cancelId);

      /** 
       *  Use this method when you need not tick stream requested by requestTickStream method.
       *  
       *  @param cancelId  Represents request id returned by requestTickStream
       */
      void cancelTickStream (int cancelId);

      /** 
       *  Use this method when you need not tick stream requested by requestOptionChain method.
       *  
       *  @param cancelId  Represents request id returned by requestOptionChain
       */
      void cancelOptionChain (int cancelId);

      /** 
       *  Use this method when you need not tick stream requested by requestBookStream method.
       *  
       *  @param cancelId  Represents request id returned by requestBookStream
       */
      void cancelBookStream (int cancelId);            

      /** 
       *  Use this method when you need not historical stream requested by requestHist*.
       *  
       *  @param cancelId  Represents request id returned by requestHistData, requestHistTicks requestHistBooks
       */
      void cancelHistData (int cancelId);
      
      /** 
       *  Attempts to log into the server using specified username and password, 
       *  using network options previously set by addHost.
       *  
       *  @param username  It is your login. You can get it on http://www.opentick.com
       *  @param password  It is your password
       *  @see addHost
       *  @see logout
       */
      void login (char *username, char *password);

      /** 
       *  Logs out and closes the connection.
       *  
       */
      void logout();

      /** 
       *  This function, as well as clearHosts are used to manage the list of network
       *  addresses where instances of the opentick server are running at. This function
       *  adds one network address to the list. The otFeed client API allows to set several
       *  servers via this routine. It is desirable that you specify as many hosts as possible.
       *  
       *  @param hostName  IP address or domain name of the feed
       *  @param portNum  Port of the feed
       *  
       *  @code addHost("feed1.opentick.com", 10010);  
       *  @endcode 
       */
      void addHost (char *hostName, int portNum);

      /** 
       *  Clears the entire list of network addresses/ports otFeed will attempt to connect to.
       *  
       */
      void clearHosts();

      /** 
       *  Returns a value which indicates whether you are logged in or not.
       *  
       *  @return  Value which indicates whether you are logged in or not. 
       */
      bool isLoggedIn();

      /** 
       *  Returns an integer value which indicates the current status of the OTClient object.
       *  
       *  @return  Integer value which indicates the current status of the OTClient object. 
       */
      int getStatus();

      /** 
       *  Use this method if you want to specify your platform. Call of this method is 
       *  optional. However, if you call it then you must do that before login call.
       *  
       *  @param platformId  Application's id
       *  @param platformIdPwd  Password for confirmation application's id
       */
      void setPlatformId (int platformId, char *platformIdPwd);

      /** 
       *  Use this method to interpret request id by symbol and exchange you
       *  have specified in request. This function could be useful in  event handlers.
       *  
       *  @param reqId  Id of the request
       *  @param entity  Resulting entity if request id is valid
       *  @return Returns true if request id is valid, false - otherwise 
       */
      bool getEntityById(int reqId, OTDataEntity& entity);

      /** 
       *  This method is called when the client successfully logs in.
       *  
       */
      virtual void onLogin();

      /** 
       *  This method is called when some error occurs.
       *  
       *  @param error  Event data
       */
      virtual void onError (OTError *error);

      /** 
       *  This method is called when some message is sent to the user.
       *  
       *  @param message  Event data
       */
      virtual void onMessage (OTMessage *message);

      /** 
       *  Provides list of exchanges.
       *  
       *  @param exchanges  List of exchanges
       *  @param exchangeCount  Number of exchanges in list
       */
      virtual void onListExchanges (OTExchange *exchanges, int exchangeCount);

      /** 
       *  Provides list of symbols.
       *  
       *  @param symbols  Part of symbols list
       *  @param symbolCount  Number of symbols in list
       */
      virtual void onListSymbols (OTSymbol *symbols, int symbolCount);

      /** 
       *  Provides a real-time level I quote.
       *  
       *  @param quote  Event data
       */
      virtual void onRealtimeQuote (OTQuote *quote);

      /** 
       *  Provides a real-time level II quote.
       *  
       *  @param mmQuote  Event data
       */
      virtual void onRealtimeMMQuote (OTMMQuote *mmQuote);

      /** 
       *  Provides a real-time trade.
       *  
       *  @param trade  Event data
       */
      virtual void onRealtimeTrade (OTTrade *trade);

      /** 
       *  Provides a real-time level I quote (half of it).
       *  
       *  @param bbo  Event data
       */
      virtual void onRealtimeBBO (OTBBO *bbo);

      /** 
       *  Provides an Open, High and Low values for the current day.
       *  All values are set to zero if there weren’t any trades in the current day.
       *  
       *  @param requestId  Id of the request
       *  @param openPrice  Today's open price
       *  @param highPrice  Today's high price
       *  @param lowPrice  Today's low price
       */
      virtual void onTodaysOHL (int requestId, double openPrice, double highPrice, double lowPrice);

      /** 
       *  Provides a historical level I quote.
       *  
       *  @param quote  Event data
       */
      virtual void onHistQuote (OTQuote *quote);

      /** 
       *  Provides a historical level II quote.
       *  
       *  @param mmQuote  Event data
       */
      virtual void onHistMMQuote (OTMMQuote *mmQuote);

      /** 
       *  Provides a historical trade.
       *  
       *  @param trade  Event data
       */
      virtual void onHistTrade (OTTrade *trade);

      /** 
       *  Provides a historical level I quote (half of it).
       *  
       *  @param bbo  Event data
       */
      virtual void onHistBBO (OTBBO *bbo);

      /** 
       *  Provides historical market data (OHLC values).
       *  
       *  @param data  Event data
       */
      virtual void onHistOHLC (OTOHLC *data);

      /** 
       *  This method is called when the connection is restored after being
       *  broken for some reason.
       *  
       */
      virtual void onRestoreConnection();

      /** 
       *  This method is called when the status changes.
       *  
       *  @param status  New status of the API
       */
      virtual void onStatusChanged (int status);

      /** 
       *  Provides real-time Equity Init data. This is various information 
       *  about company and their listed stock.
       *  
       *  @param equityInit  Event data
       */
      virtual void onEquityInit (OTEquityInit *equityInit);

      /** 
       *  Provides a real-time Book Cancel data.
       *  
       *  @param bookCancel  Event data
       */
      virtual void onBookCancel (OTBookCancel *bookCancel);

      /** 
       *  Provides a real-time Book Change data.
       *  
       *  @param bookChange  Event data
       */
      virtual void onBookChange (OTBookChange *bookChange);   

      /** 
       *  Provides a real-time Book Delete data.
       *  
       *  @param bookDelete  Event data
       */
      virtual void onBookDelete (OTBookDelete *bookDelete);   

      /** 
       *  Provides a real-time Book Execute data.
       *  
       *  @param bookExecute  Event data
       */
      virtual void onBookExecute (OTBookExecute *bookExecute);  

      /** 
       *  Provides a real-time Book Order data.
       *  
       *  @param bookOrder  Event data
       */
      virtual void onBookOrder (OTBookOrder *bookOrder);    

      /** 
       *  Provides a real-time Book Price Level data.
       *  
       *  @param bookPriceLevel  Event data
       */
      virtual void onBookPriceLevel (OTBookPriceLevel *bookPriceLevel);

      /** 
       *  Provides a real-time Book Purge data.
       *  
       *  @param bookPurge  Event data
       */
      virtual void onBookPurge (OTBookPurge *bookPurge);    

      /** 
       *  Provides a real-time Book Replace data.
       *  
       *  @param bookReplace  Event data
       */
      virtual void onBookReplace (OTBookReplace *bookReplace);

      /** 
       *  This method is called upon receipt of an OTSplit event.
       *  
       *  @param split  Event data
       */
      virtual void onSplit (OTSplit *split);

      /** 
       *  This method is called upon receipt of an OTDividend event.
       *  
       *  @param dividend  Event data
       */
      virtual void onDividend(OTDividend *dividend);

      /** 
       *  Provides a historical Book Cancel data.
       *  
       *  @param bookCancel  Event data
       */
      virtual void onHistBookCancel (OTBookCancel *bookCancel);

      /** 
       *  Provides a historical Book Change data.
       *  
       *  @param bookChange  Event data
       */
      virtual void onHistBookChange (OTBookChange *bookChange);   

      /** 
       *  Provides a historical Book Delete data.
       *  
       *  @param bookDelete  Event data
       */
      virtual void onHistBookDelete (OTBookDelete *bookDelete);   

      /** 
       *  Provides a historical Book Execute data.
       *  
       *  @param bookExecute  Event data
       */
      virtual void onHistBookExecute (OTBookExecute *bookExecute);  

      /** 
       *  Provides a historical Book Order data.
       *  
       *  @param bookOrder  Event data
       */
      virtual void onHistBookOrder (OTBookOrder *bookOrder);    

      /** 
       *  Provides a historical Book Price Level data.
       *  
       *  @param bookPriceLevel  Event data
       */
      virtual void onHistBookPriceLevel (OTBookPriceLevel *bookPriceLevel);

      /** 
       *  Provides a historical Book Purge data.
       *  
       *  @param bookPurge  Event data
       */
      virtual void onHistBookPurge (OTBookPurge *bookPurge);    

      /** 
       *  Provides a historical Book Replace data.
       *  
       *  @param bookReplace  Event data
       */
      virtual void onHistBookReplace (OTBookReplace *bookReplace);

      /** 
       *  This method is called upon receipt of an Option Initialize event.
       *  
       *  @param optionInit  Event data
       */
      virtual void onOptionInit (OTOptionInit *optionInit);

    private:
      OTClient (const OTClient &);
      OTClient & operator = (const OTClient &);

      int requestOption (char requestType, OTDataEntity *dataEntity, int expYear, short int expMonth, double minStrikePrice, double maxStrikePrice, int paramsType, int mask = 0);
      int requestSnapshot (OTDataEntity *entity, int snapshotMask);
      int requestTicks (OTDataEntity *dataEntity, int mask);

      void addToRequestsVector(OTDataEntity* dataEntity, int requestId);   
      void delFromRequestsVector(int requestId);   
      int findEntityById(int requestId);
    
      void sendCommand (OTCommand *command);
      void recvResponses();
      void pollResponses();

      bool startRecvResponses();
      bool startPollResponses();  
      void stopRecvResponses();
      void stopPollResponses();

      void startThreads();
      void stopThreads();
      void clearContainers();

      void processResponse (int commandId, int reqId, bool statusOk, int errCode, char *data);
      void restoreConnection();
      void reRequest();
      void removeFromActive (int reqId);
      void notifyOfError (int requestId, int type, int code, char *description);

      void onOTLogin (char *, int);
      void onOTLogout (char *, int);
      void onOTTickStream (char *, int, bool);
      void onOTCancel (char *data, int requestId, int cancelCode, const char* cancelDescription);
      void onOTResponseHistData (char *, int);
      void onOTResponseListExchanges (char *, int);
      void onOTResponseListSymbols (char *, int);
      void onOTEquityInit (char *, int);
      void onOTBookStream (char *, int);
      void onOTError (char *, int);
      void onOTSplit (char *, int);
      void onOTDividend (char *, int);
      void onOTResponseHistBooks(char *, int);
      void onOTResponseOptionInit(char *, int);
 
      static THREAD_FUNC_RETVAL recvResponsesWrapper (void *);
      static THREAD_FUNC_RETVAL pollResponsesWrapper (void *);

      void setStatus (int status);
      bool isCriticalMsg (char *);

      void getNextAddress();
      void clearAddressList();
      void addServerInfo (void *);

      // members
      OTSocket  *socket;
      char      *sessionId;
      int        requestId;
      int        currStatus;
      int        targetStatus;

      // os & platform id
      int    osId;
      int    platformId;
      char  *platformIdPwd;

      // load balancing 
      std::vector<void*> serverList;
      int   runServer;
      int   redirectCount;
      bool  isRedirect;

      //used in requestMarketDepth and canselMarketDepth
      std::vector<int> marketDepthBooks;
    
      // flags
      int  manualLogin;
      bool reRequested;

      bool runsRecvResponses;
      bool runsPollResponses;
      bool isLogin;

      // objects used to return data to user
      OTQuote       quote;
      OTMMQuote     mmQuote;
      OTTrade       trade;
      OTOHLC        ohlc;
      OTExchange   *listExchanges;
      OTSymbol     *listSymbols;
      OTMessage     message;
      OTError       error;
      OTEquityInit  equityInit;

      OTBookCancel     bookCancel;
      OTBookChange     bookChange;
      OTBookDelete     bookDelete;
      OTBookExecute    bookExecute;
      OTBookOrder      bookOrder;
      OTBookPriceLevel bookPriceLevel;
      OTBookPurge      bookPurge;
      OTBookReplace    bookReplace;
      OTBBO            bbo;
      OTSplit          split;
      OTDividend       dividend; 
      OTOptionInit     opInit;
      // vector of requested DataEntity
      std::vector<RequestsVectorElement_t> requestsVector;

      // requests, responses, commands
      VectorCancelRequest  *cancelRequests;
      VectorRequest        *activeRequests;
      QueueResponse        *pendingResp;              
      OTLoginCmd           *loginCmd;

      // thread handles
      FThread hRecvResponses;
      FThread hPollResponses;

      // events
      Event evNewResponse;

      // critical sections
      CriticalSection csRequestsVector;
      CriticalSection csCancelRequests;
      CriticalSection csActiveRequests;
      CriticalSection csPendingResp;
      CriticalSection csLogin;
      CriticalSection csRestoreConnection;
      CriticalSection csRecvResponses;
      CriticalSection csPollResponses;
      CriticalSection csUserFunc;
      CriticalSection csHostList;
      CriticalSection csStatus;
  };

#endif
