/** \file OTCommands.h
*   \brief  OTCommand, OTLoginCmd, OTLogoutCmd, OTRequest, OTRequestHistData, 
*           OTRequestTickStream, OTHeartBeatCmd, OTRequestListExchanges,
*           OTRequestListSymbols classes definitions
*
*   Copyright (C) 2004,2005 Opentick corporation. All rights reserved.
*/


#ifndef _OT_COMMANDS_H
#define _OT_COMMANDS_H

#include <vector>
#include "OTDataEntity.h"


  /** This is the abstract base class of the "OTCommand" hierarchy. \n
  *   It is used to disengage from the implementation of wrapToOTP function
  *   for various types of OTP commands.
  */
  class OTCommand 
  {
    public:
      OTCommand (int &id, bool cancelable);
      virtual ~OTCommand ();
      virtual char *wrapToOTP (char *, int &) = 0;
      int getId ();
      bool getCancelable();

    protected:
      OTCommand (const OTCommand &);
      OTCommand & operator = (const OTCommand &);

      int id;
      bool cancelable;
  };



  /** This class is used to store parameters of the OTLogin command.
  */  
  class OTLoginCmd: public OTCommand 
  {
    public:
      OTLoginCmd (int &id, int osId, int platformId, char * platformPwd, char *username, char *password);
      virtual ~OTLoginCmd ();
      virtual char *wrapToOTP (char *, int &);

    private:
      OTLoginCmd (const OTLoginCmd &);
      OTLoginCmd & operator = (const OTLoginCmd &);

      char *username;
      char *password;
      char *platformPwd;
      int  osId;
      int  platformId;
  };



  /** This class is used to store parameters of the OTLogout command.
  */  
  class OTLogoutCmd: public OTCommand 
  {
    public:
      OTLogoutCmd (int &id);
      virtual ~OTLogoutCmd ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTLogoutCmd (const OTLogoutCmd &);
      OTLogoutCmd & operator = (const OTLogoutCmd &);

  };



  /** This is the abstract base class of the "OTRequest" hierarchy. \n
  *   It is used to disengage from the implementation of wrapToOTP function
  *   for various types of OTP OTRequest* commands.
  */
  class OTRequest: public OTCommand 
  {
    public:
      OTRequest (int &id, bool cancelable);
      virtual ~OTRequest ();
      virtual void updateData (int) {}

    private:
      OTRequest (const OTRequest &);
      OTRequest & operator = (const OTRequest &);
  };



  /** This class is used to store parameters of the OTRequestHistData command.
  */
  class OTRequestHistData: public OTRequest 
  {
    public:
      OTRequestHistData (int &id,
                         OTDataEntity *dataEntity,
                         int startDate, 
                         int endDate, 
                         int dataType,
                         int interval);

      virtual ~OTRequestHistData ();
      virtual char *wrapToOTP (char *sessionId, int &);
      virtual void updateData(int);

    private:
      OTRequestHistData (const OTRequestHistData &);
      OTRequestHistData & operator = (const OTRequestHistData &);

      OTDataEntity *dataEntity;
      int startDate;
      int endDate;
      char dataType;
      short interval;
  };


 /** This class is used to store parameters of the OTRequestHistTicks command.
  */
 class OTRequestHistTicks: public OTRequest 
  {
    public:
      OTRequestHistTicks(int &id,
                         OTDataEntity *dataEntity,
                         int startDate, 
                         int endDate, 
                         int mask);

      virtual ~OTRequestHistTicks ();
      virtual char *wrapToOTP (char *sessionId, int &);
      virtual void updateData(int);

    private:
      OTRequestHistTicks (const OTRequestHistTicks &);
      OTRequestHistTicks & operator = (const OTRequestHistTicks &);

      OTDataEntity *dataEntity;
      int startDate;
      int endDate;
      int mask;
  };


 /** This class is used to store parameters of the OTRequestHistBooks command.
  */
 class OTRequestHistBooks: public OTRequest 
  {
    public:
      OTRequestHistBooks(int &id,
                         OTDataEntity *dataEntity,
                         int startDate, 
                         int endDate, 
                         int mask);

      virtual ~OTRequestHistBooks ();
      virtual char *wrapToOTP (char *sessionId, int &);
      virtual void updateData(int);

    private:
      OTRequestHistBooks (const OTRequestHistBooks &);
      OTRequestHistBooks & operator = (const OTRequestHistBooks &);

      OTDataEntity *dataEntity;
      int startDate;
      int endDate;
      int mask;
  };


  /** This class is used to store parameters of the OTRequestTickStream command.
   */
  class OTRequestTickStream: public OTRequest
  {
   public:
     OTRequestTickStream (int &id, OTDataEntity *dataEntity, int mask);
     virtual ~OTRequestTickStream (); 
     virtual char *wrapToOTP (char *sessionId, int &);

   private:
     OTRequestTickStream (const OTRequestTickStream &);
     OTRequestTickStream & operator = (const OTRequestTickStream &);

     OTDataEntity *dataEntity; 
     int mask;
  };
      



  /** This class is used to store parameters of the OTRequestEquityInit command.
  */
  class OTRequestEquityInit: public OTRequest
  {
    public:
      OTRequestEquityInit (int &id, OTDataEntity *dataEntity);
      virtual ~OTRequestEquityInit();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestEquityInit (const OTRequestEquityInit &);
      OTRequestEquityInit & operator = (const OTRequestEquityInit &);

      OTDataEntity *dataEntity;
  };


  /** This class is used to store parameters of the extended OTRequestBookStream command.
  */
  class OTRequestBookStream: public OTRequest 
  {
    public:
      OTRequestBookStream (int &id, OTDataEntity *dataEntity, int mask);
      virtual ~OTRequestBookStream ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestBookStream (const OTRequestBookStream &);
      OTRequestBookStream & operator = (const OTRequestBookStream &);

      OTDataEntity *dataEntity;
      int mask;
  };




  /** This class is used to store parameters of the universal OTRequestOption command.
  */
  class OTRequestOption: public OTRequest
  {
    public:
      OTRequestOption (char requestType, int &id, OTDataEntity *dataEntity, int expYear, short int expMonth, double minStrikePrice, double maxStrikePrice, int type, int mask);
      virtual ~OTRequestOption();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestOption (const OTRequestOption &);
      OTRequestOption & operator = (const OTRequestOption &);

      OTDataEntity *dataEntity;
      int expYear;
      short int expMonth;
      double minStrikePrice; 
      double maxStrikePrice;
      int type;
      int mask;
      char requestType;
  };
  


  /** This class is used to store parameters of the OTRequestTickSnapshot
  */
  class OTRequestTickSnapshot: public OTRequest
  {
    public:
      OTRequestTickSnapshot (int &id, OTDataEntity *dataEntity, int snapshotMask);
      virtual ~OTRequestTickSnapshot();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestTickSnapshot (const OTRequestTickSnapshot &);
      OTRequestTickSnapshot & operator = (const OTRequestTickSnapshot &);

      OTDataEntity *dataEntity;
      int snapshotMask;
  };



  /** This class is used to store parameters of the OTHeartbeat command.
  */
  class OTHeartbeatCmd: public OTCommand 
  {
    public:
      OTHeartbeatCmd (int &id);
      virtual ~OTHeartbeatCmd();
      virtual char *wrapToOTP (char *, int &);

    private:
      OTHeartbeatCmd (const OTHeartbeatCmd &);
      OTHeartbeatCmd & operator = (const OTHeartbeatCmd &);
      
  };



  /** This class is used to generate OTRequestListExchanges OTP request.
  */
  class OTRequestListExchanges: public OTRequest 
  {
    public:
      OTRequestListExchanges (int &id);
      virtual ~OTRequestListExchanges ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestListExchanges (const OTRequestListExchanges &);
      OTRequestListExchanges & operator = (const OTRequestListExchanges &);
  };


  
  /** This class is used to generate OTRequestListSymbols OTP request.
  */
  class OTRequestListSymbols: public OTRequest 
  {
    public:
      OTRequestListSymbols (int &id, const char *exchange, const char* symbol, int options);
      virtual ~OTRequestListSymbols ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestListSymbols (const OTRequestListSymbols &);
      OTRequestListSymbols & operator = (const OTRequestListSymbols &);

      char *exchange;
      char *symbol;
      int options;
  };

  /** This class is used to generate OTRequestSplits OTP request.
  */
  class OTRequestSplits: public OTRequest 
  {
    public:
      OTRequestSplits (int &id, OTDataEntity *dataEntity, int startingDate, int endingDate);
      virtual ~OTRequestSplits ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestSplits (const OTRequestSplits &);
      OTRequestSplits & operator = (const OTRequestSplits &);

      OTDataEntity *dataEntity;
      int startingDate;
      int endingDate;
  };


  /** This class is used to generate OTRequestDividends OTP request.
  */
  class OTRequestDividends: public OTRequest 
  {
    public:
      OTRequestDividends (int &id, OTDataEntity *dataEntity, int startingDate, int endingDate);
      virtual ~OTRequestDividends ();
      virtual char *wrapToOTP (char *sessionId, int &);

    private:
      OTRequestDividends (const OTRequestDividends &);
      OTRequestDividends & operator = (const OTRequestDividends &);

      OTDataEntity *dataEntity;
      int startingDate;
      int endingDate;
  };



  typedef std::vector <OTRequest *> VRequest;
  typedef VRequest::iterator RequestIter;


  /** This class is used to store pointers to objects of the OTRequest type
  *   in the standard vector container.
  */
  class VectorRequest : public VRequest 
  {
    public:
      virtual ~VectorRequest () 
      {
        clear();
      }

      /** This function is used to search the vector for an object of the Request 
      *   type by the given requestId.
      *   \param id (input) - requestId to search by;
      *   \return iterator which points to the object (if it has been found) \b or \n
      *           iterator which points to the end() of the vector (otherwise).
      */
      RequestIter findById (int id) 
      {
        for (RequestIter it = begin (); it != end (); it++) 
        {
          if ((*it)->getId() == id) 
          {
            return it;
          }
        }
        return end ();
      }
         
      /** This function is used to empty the vector.
        \return none.
      */
      virtual void clear () 
      {
        for (RequestIter it = begin (); it != end (); it++) 
        {
          delete *it;
        }
        VRequest::clear();
      }
    
      /** This function is used to remove element from the vector.
          \param it (input) - iterator which points to element to remove;
          \return none.
      */
      void erase (RequestIter it) 
      {
        delete *it;
        VRequest::erase (it);
      }
  };



  /** This class is used to store parameters of the OTCancel* command,
  *  and to generate an appropriate OTP request.
  */
  class OTCancelRequest: public OTCommand 
  {
    public:
      OTCancelRequest (int &id, OTRequest *req, int cancelId,  int commandType);
      virtual ~OTCancelRequest ();
      virtual char *wrapToOTP (char *sessionId, int &);
      int getCancelId ();

    private:
      OTCancelRequest (const OTCancelRequest &);
      OTCancelRequest & operator = (const OTCancelRequest &);

      int cancelId;
      int commandType;
  };



  typedef std::vector <OTCancelRequest *> VCancelRequest;
  typedef VCancelRequest::iterator CancelRequestIter;

  /** This class is used to store pointers to objects of the CancelRequest type
  *   in the standard vector container.
  */
  class VectorCancelRequest : public VCancelRequest 
  {
    public:
      virtual ~VectorCancelRequest () 
      {
        clear();
      }

      /** This function is used to search the vector for an object of the CancelRequest 
      *   type by the given requestId.
      *   \param id (input) - requestId to search by;
      *   \return iterator which points to the object (if it has been found) \b or \n
      *           iterator which points to the end() of the vector (otherwise).
      */
      CancelRequestIter findById (int id) 
      {
        for (CancelRequestIter it = begin ();  it != end (); it++) 
        {
          if ((*it)->getId() == id) 
          {
            return it;
          }
        }
        return end ();
      }

      /** This function is used to search the vector for an object of the CancelRequest 
      *   type by the given cancelId.
      *   \param cancelId (input) - cancelId to search by;
      *   \return iterator which points to the object (if it has been found) \b or \n
      *           iterator which points to the end() of the vector (otherwise).
      */
      CancelRequestIter findByCancelId (int cancelId) 
      {
        for (CancelRequestIter it = begin ();  it != end (); it++) 
        {
          if ((*it)->getCancelId() == cancelId) 
          {
            return it;
          }
        }
        return end ();
      }

      /** This function is used to empty the vector.
        \return none.
      */
      virtual void clear () 
      {
        for (CancelRequestIter it = begin (); it != end (); it++) 
        {
          delete *it;
        }
        VCancelRequest::clear();
      }

      /** This function is used to remove element from the vector.
        \param it (input) - iterator which points to element to remove;
        \return none.
      */
      virtual void erase (CancelRequestIter it) 
      {
        delete *it;
        VCancelRequest::erase (it);
      }
  };


#endif
