/** \file OTEquityInit.h
*   \brief OTEquityInit class definition
*
*   Copyright (C) 2004,2005 Opentick corporation. All rights reserved.
*/

#ifndef _OT_EQUITYINIT_H
#define _OT_EQUITYINIT_H


#include "OTDef.h"

  /// <summary>
  /// Provides Equity Initialize information. This is information about 
  /// a company and it’s stock.
  /// </summary>
  class   OTEquityInit
  {
    public:
      /// <summary>Default constructor.</summary>
      OTEquityInit();

      /// <summary>Destructor</summary>
      ~OTEquityInit();

      /// <returns>ID of the appropriate request.</returns>
      int      getRequestId();

      /// <returns> Instrument type.</returns>
      int      getInstrumentType();

      /// <returns> Notes whether Nasdaq flagged as UPC11830.</returns>
      bool     isUPC11830();

      /// <returns> Notes whether it is a Nasdaq Small Cap.</returns>
      bool     isSmallCap();

      /// <returns> Notes whether it is a known test symbol.</returns>
      bool     isTestlssue();

      /// <returns> Currency.</returns>
      char    *getCurrency();

      /// <returns>Company name.</returns>
      char    *getCompanyName();

      /// <returns>Previous close date: YYYYMMDD or 8 spaces if not provided.</returns>
      char    *getPrevCloseDate();

      /// <returns> Annual high date: YYYYMMDD or 8 spaces if not provided.</returns>
      char    *getAnnualHighDate();

      /// <returns> Annual low date: YYYYMMDD or 8 spaces if not provided.</returns>
      char    *getAnnualLowDate();

      /// <returns> Earnings date: YYYYMMDD or 8 spaces if not provided.</returns>
      char    *getEarningsDate();

      /// <returns> CUSIP number.</returns>
      char    *getCUSIP();

      /// <returns> International Securities Identification Number, or ISIN.</returns>
      char    *getISIN();

      /// <returns>Previous close price.</returns>
      double   getPrevClosePrice();

      /// <returns>Annual high price.</returns>
      double   getAnnualHighPrice();

      /// <returns> Annual low price.</returns>
      double   getAnnualLowPrice();

      /// <returns> Earnings price (earnings per share).</returns>
      double   getEarningsPrice();      

      /// <returns> Total shares.</returns>
      __int64  getTotalShares();

      /// <returns> Average volume.</returns>
      __int64  getAverageVolume();

      /// <summary>
      /// Sets ID of the appropriate request.
      /// </summary>
      /// <param name="requestId">ID of the appropriate request.</param>
      void setRequestId (int requestId);

      /// <summary>
      /// Sets instrument type.
      /// </summary>
      /// <param name="instrumentType">Instrument type</param>
      void setInstrumentType (int instrumentType); 

      /// <summary>
      /// Sets currency.
      /// </summary>
      /// <param name="currency">Currency.</param>
      void setCurrency (char *currency);

      /// <summary>
      /// Sets company name.
      /// </summary>
      /// <param name="companyName">Company name.</param>
      void setCompanyName (char *companyName);

      /// <summary>
      /// Sets previous close date.
      /// </summary>
      /// <param name="prevCloseDate">Previous close date.</param>
      void setPrevCloseDate (char *prevCloseDate);

      /// <summary>
      /// Sets annual high date.
      /// </summary>
      /// <param name="annualHighDate">Annual high date</param>
      void setAnnualHighDate (char *annualHighDate);

      /// <summary>
      /// Sets annual low date.
      /// </summary>
      /// <param name="annualLowDate">Annual low date</param>
      void setAnnualLowDate (char *annualLowDate);

      /// <summary>
      /// Sets earnings date.
      /// </summary>
      /// <param name="earningsDate">Earnings date.</param>
      void setEarningsDate (char *earningsDate);

      /// <summary>
      /// Sets CUSIP number.
      /// </summary>
      /// <param name="cusip">CUSIP number.</param>
      void setCUSIP (char *cusip); 

      /// <summary>
      /// Sets International Securities Identification Number, or ISIN.
      /// </summary>
      /// <param name="isin">International Securities Identification Number, or ISIN.</param>
      void setISIN (char *isin);

      /// <summary>
      /// Sets previous close price.
      /// </summary>
      /// <param name="prevClosePrice">Previous close price.</param>
      void setPrevClosePrice (double prevClosePrice);

      /// <summary>
      /// Sets annual high price.
      /// </summary>
      /// <param name="annualHighPrice">Annual high price.</param>
      void setAnnualHighPrice (double annualHighPrice);

      /// <summary>
      /// Sets annual low price.
      /// </summary>
      /// <param name="annualLowPrice">Annual low price.</param>
      void setAnnualLowPrice (double annualLowPrice);

      /// <summary>
      /// Sets earnings price.
      /// </summary>
      /// <param name="earningsPrice">Earnings price.</param>
      void setEarningsPrice (double earningsPrice);

      /// <summary>
      /// Sets total shares.
      /// </summary>
      /// <param name="totalShares">Total shares.</param>
      void setTotalShares (__int64 totalShares);

      /// <summary>
      /// Sets average volume.
      /// </summary>
      /// <param name="averageVolume">Average volume.</param>
      void setAverageVolume (__int64 averageVolume);

      /// <summary>
      /// Adds UPC118300 bit.
      /// </summary>
      /// <param name="isUPC11830">Notes whether Nasdaq flagged as UPC11830.</param>
      void setIsUPC11830  (bool isUPC11830);

      /// <summary>
      /// Adds Small Cap bit.
      /// </summary>
      /// <param name="isSmallCap">Notes whether it is a Nasdaq Small Cap.</param>
      void setIsSmallCap  (bool isSmallCap);

      /// <summary>
      /// Adds test symbol bit.
      /// </summary>
      /// <param name="isTestlssue">Notes whether it is a known test symbol.</param>
      void setIsTestlssue (bool isTestlssue);
           
    private:
      OTEquityInit (const OTEquityInit &);
      OTEquityInit & operator = (const OTEquityInit &);

      int      requestId;
      int      instrumentType;
      bool     isUPC11830Field;
      bool     isSmallCapField;
      bool     isTestlssueField;
      char    *currency;
      char    *companyName;
      char    *prevCloseDate;
      char    *annualHighDate;
      char    *annualLowDate;
      char    *earningsDate;
      char    *cusip;
      char    *isin;
      double   prevClosePrice;
      double   annualHighPrice;
      double   annualLowPrice;
      double   earningsPrice;
      __int64  totalShares;
      __int64  averageVolume;

  };

#endif

