/** \file OTMMQuote.h
*   \brief OTMMQuote class definition.
*
*   Copyright (C) 2004 Opentick corporation. All rights reserved.
*/

#ifndef _OT_MMQUOTE_H
#define _OT_MMQUOTE_H

#include "OTDef.h"

  /// <summary>
  /// This class provides the level II quote provided by a market maker for
  /// NASDAQ equities, or regional quotes for listed stocks. The market maker
  /// ID (MMID) identifies which specific market participant has changed his
  /// quote. In the case of NASDAQ, the MMID is a string identifying the market
  /// maker. From SIAC and OPRA, the quotes are from regional exchanges, and 
  /// the MMID identifies the exchange that the regional quote comes from. 
  /// </summary>
  class   OTMMQuote 
  {
    public:
      /// <summary>Default constructor.</summary>
      OTMMQuote ();

      /// <summary> Destructor. </summary>
      ~OTMMQuote ();

      /// <returns>ID of the appropriate request.</returns>
      int     getRequestId(); 

      /// <returns>Time when the event occurred.</returns>
      int     getTimestamp();            

      /// <returns>Number of round lots in the ask.</returns>
      int     getAskSize();

      /// <returns>Number of round lots in the bid.</returns>
      int     getBidSize();

      /// <returns> Market Maker ID.</returns>
      char   *getMMID();

      /// <returns>Indicator.</returns>
      char    getIndicator();

      /// <returns> Symbol code; this field is provided for option chains only, 
      /// because there can be more than one symbol appropriate to the requested underlyer and expiry date.
      /// </returns>
      char   *getSymbol();

      /// <returns>Exchange code.</returns>
      char   *getExchange();

      /// <returns>Ask price.</returns>
      double  getAskPrice();

      /// <returns>Bid price.</returns>
      double  getBidPrice();

      /// <summary>
      /// Sets ID of the appropriate request.
      /// </summary>
      /// <param name="requestId">ID of the appropriate request.</param>
      void setRequestId (int requestId);

      /// <summary>Sets time of the event.</summary>
      /// <param name="timestamp">Time when the event occurred.</param>
      void setTimestamp (int timestamp);

      /// <summary>
      /// Sets ask price.
      /// </summary>
      /// <param name="value">Ask price.</param>
      void setAskPrice (double value);

      /// <summary>
      /// Sets bid price.
      /// </summary>
      /// <param name="value">Bid price.</param>
      void setBidPrice (double value);      

      /// <summary>
      /// Sets number of round lots in the ask.
      /// </summary>
      /// <param name="value">Number of round lots in the ask.</param>
      void setAskSize (int value);

      /// <summary>
      /// Sets number of round lots in the bid.
      /// </summary>
      /// <param name="value">Number of round lots in the bid.</param>
      void setBidSize (int value);

      /// <summary>
      /// Sets market maker id.
      /// </summary>
      /// <param name="mmid"> Market Maker ID.</param>
      void setMMID (char *mmid);

      /// <summary>
      /// Sets indicator.
      /// </summary>
      /// <param name="indicator">Indicator.</param>
      void setIndicator (char indicator);

      /// <summary>
      /// Symbol code.
      /// </summary>
      /// <param name="symbol">Symbol code.</param>
      void setSymbol (char *symbol);

      /// <summary>
      /// Sets exchange code.
      /// </summary>
      /// <param name="exchange">Exchange code.</param>
      void setExchange (char *exchange);

    private:
      OTMMQuote (const OTMMQuote &);
      OTMMQuote & operator = (const OTMMQuote &);

      int     requestId;
      int     timestamp;
      int     askSize;
      int     bidSize;
      char    mmid[5];
      char    indicator;
      char   *symbol;
      char    exchange[3];
      double  askPrice;
      double  bidPrice;

  };


#endif
