/** \file OTOHLC.h
*   \brief OTOHLC class definition.
*
*   Copyright (C) 2004,2005 Opentick corporation. All rights reserved.
*/

#ifndef _OT_OHLC_H
#define _OT_OHLC_H

#include "OTDef.h"


  /// <summary>
  /// This class provides historical OHLC values (open, high, low, and close prices) for the requested time period.
  /// Also it provides the volume value (the accumulated number of shares) in the requested time period. In cases when 
  /// there were not any transactions within certain time period, all the price values, as well as the volume value, are set to zero.
  /// </summary>
  class   OTOHLC 
  {
    public:
      /// <summary> Default constructor. </summary>
      OTOHLC();

      /// <summary> Destructor. </summary>
      ~OTOHLC();

      /// <returns>ID of appropriate request.</returns>
      int     getRequestId();   

      /// <returns>Starting time of the time period.</returns>
      int     getTimestamp();

      /// <returns>Open price.</returns>
      double  getOpenPrice();

      /// <returns>High price.</returns>
      double  getHighPrice();

      /// <returns>Low price.</returns>
      double  getLowPrice();

      /// <returns>Close price.</returns>
      double  getClosePrice();

      /// <returns>Accumulated volume for the requested time period.</returns>
      __int64 getVolume();

      /// <summary>Sets ID of appropriate request.</summary>
      /// <param name="requestId">ID of appropriate request.</param>
      void setRequestId (int requestId);

      /// <summary>Sets time of the event.</summary>
      /// <param name="timestamp">Time when the event occurred.</param>
      void setTimestamp (int timestamp);

      /// <summary>Sets open price.</summary>
      /// <param name="value">Open price.</param>
      void setOpenPrice (double value);

      /// <summary>Sets high price.</summary>
      /// <param name="value">High price.</param>
      void setHighPrice (double value);

      /// <summary>Sets low price.</summary>
      /// <param name="value">Low price.</param>
      void setLowPrice  (double value);

      /// <summary>Sets close price.</summary>
      /// <param name="value">Close price.</param>
      void setClosePrice (double value);

      /// <summary>Sets accumulated volume for the requested time period.</summary>
      /// <param name="value">Accumulated volume for the requested time period.</param>
      void setVolume (__int64 value);

    private:  

      OTOHLC (const OTOHLC &);
      OTOHLC & operator = (const OTOHLC &);
      
      int     requestId;
      int     timestamp;
      __int64 volume;
      double  openPrice;
      double  highPrice;
      double  lowPrice;
      double  closePrice;
  };


#endif
