/** \file OTQuote.h
*   \brief OTQuote class definition.
*
*   Copyright (C) 2004 Opentick corporation. All rights reserved.
*/

#ifndef _OT_QUOTE_H
#define _OT_QUOTE_H

#include "OTDef.h"

  /// <summary>
  /// This class provides either a real-time or a historical level I quote from an exchange. 
  /// This comes from the exchange when the price or size of either the bid or ask changes. 
  /// Both the bid and the ask prices along with their sizes are provided in a quote.
  /// </summary>
  class   OTQuote 
  {
    public:
      /// <summary> Default constructor. </summary>
      OTQuote ();

      /// <summary> Destructor. </summary>
      ~OTQuote ();

      /// <returns>ID of the appropriate request.</returns>
      int     getRequestId(); 

      /// <returns>Time when the event occurred.</returns>
      int     getTimestamp();

      /// <returns>Number of round lots in the ask.</returns>
      int     getAskSize();

      /// <returns>Number of round lots in the bid.</returns>
      int     getBidSize();

      /// <returns>Indicator.</returns>
      char    getIndicator();

      /// <returns>Tick indicator: D - Down, U - Up, N - Not provided.</returns>
      char    getTickIndicator();

      /// <returns>Ask exchange identifier.</returns>
      char   *getAskExchange();      

      /// <returns>Bid exchange identifier.</returns>
      char   *getBidExchange();      

      /// <returns>Exchange code.</returns>
      char   *getExchange();      

      /// <returns> Symbol code; this field is provided for option chains only, 
      /// because there can be more than one symbol appropriate to the requested underlyer and expiry date.
      /// </returns>
      char   *getSymbol();

      /// <returns>Ask price.</returns>
      double  getAskPrice();

      /// <returns>Bid price.</returns>
      double  getBidPrice();

      /// <summary>
      /// Sets ID of the appropriate request.
      /// </summary>
      /// <param name="requestId">ID of the appropriate request.</param>
      void setRequestId (int requestId);

      /// <summary>Sets time of the event.</summary>
      /// <param name="timestamp">Time when the event occurred.</param>
      void setTimestamp (int timestamp);

      /// <summary>
      /// Sets ask price.
      /// </summary>
      /// <param name="value">Ask price.</param>
      void setAskPrice (double value);

      /// <summary>
      /// Sets bid price.
      /// </summary>
      /// <param name="value">Bid price.</param>
      void setBidPrice (double value);      

      /// <summary>
      /// Sets number of round lots in the ask.
      /// </summary>
      /// <param name="value">Number of round lots in the ask.</param>
      void setAskSize (int value);

      /// <summary>
      /// Sets number of round lots in the bid.
      /// </summary>
      /// <param name="value">Number of round lots in the bid.</param>
      void setBidSize (int value);

      /// <summary>
      /// Sets ask exchange identifier.
      /// </summary>
      /// <param name="askExchange">Ask exchange identifier.</param>
      void setAskExchange (char *askExchange);

      /// <summary>
      /// Sets bid exchange identifier.
      /// </summary>
      /// <param name="bidExchange">Bid exchange identifier.</param>
      void setBidExchange (char *bidExchange);

      /// <summary>
      /// Sets exchange code.
      /// </summary>
      /// <param name="exchange">Exchange code.</param>
      void setExchange (char *exchange);

      /// <summary>
      /// Sets indicator.
      /// </summary>
      /// <param name="indicator">Indicator.</param>
      void setIndicator (char indicator);

      /// <summary>
      /// Sets tick indicator: D - Down, U - Up, N - Not provided.
      /// </summary>
      /// <param name="tickIndicator">Tick indicator: D - Down, U - Up, N - Not provided.</param>
      void setTickIndicator (char tickIndicator);

      /// <summary>
      /// Symbol code.
      /// </summary>
      /// <param name="symbol">Symbol code.</param>
      void setSymbol (char *symbol);

    private:
      OTQuote (const OTQuote &);
      OTQuote & operator = (const OTQuote &);

      int     requestId;
      int     timestamp;
      int     askSize;
      int     bidSize;
      char    askExchange[3];
      char    bidExchange[3];
      char    exchange[3];    
      char    indicator;
      char    tickIndicator; 
      char   *symbol;
      double  askPrice;
      double  bidPrice;

  };


#endif
