/** \file OTTrade.h
*   \brief OTTrade class definition.
*
*   Copyright (C) 2004 Opentick corporation. All rights reserved.
*/

#ifndef _OT_TRADE_H
#define _OT_TRADE_H

#include "OTDef.h"

  /// <summary>
  /// Provides either real-time or historical trade information. 
  /// This event indicates that shares were sold and purchased (excepting some tracing messages). 
  /// OTTrade stores price, size and other information about the deal.
  /// </summary>
  class   OTTrade 
  {
    public:
      /// <summary> Default constructor. </summary>
      OTTrade ();

      /// <summary> Destructor. </summary>
      ~OTTrade ();

      /// <returns>ID of appropriate request.</returns>
      int      getRequestId(); 

      /// <returns>Time when the event occurred.</returns>
      int      getTimestamp();

      /// <returns>Number of shares traded in this transaction.</returns>
      int      getSize();

      /// <returns>Sequence number of trade; used to correct trades in cancellations.</returns>
      int      getSequenceNumber();

      /// <returns>Indicator: See the Trade Indicators section for details.</returns>
      char     getIndicator();

      /// <returns>Tick indicator: D - Down, U - Up, N - Not provided.</returns>
      char     getTickIndicator();

      /// <returns>Symbol code; 
      /// this field is provided for option chains only, 
      /// because there can be more than one symbol appropriate to the requested underlyer and expiry date.</returns>
      char    *getSymbol();

      /// <returns>Exchange code.</returns>
      char    *getExchange();

      /// <returns>Notes whether the trade provides the opening price for the day.</returns>
      bool     isOpen();

      /// <returns>Notes whether the trade provides the high price for the day.</returns>
      bool     isHigh();

      /// <returns>Notes whether the trade provides the low price for the day.</returns>
      bool     isLow();

      /// <returns>Notes whether the trade provides the closing price for the day.</returns>
      bool     isClose();

      /// <returns>Notes whether the trade updates the last trade price.</returns>
      bool     isUpdateLast();

      /// <returns>Notes whether the volume is a replacement consolidated volume.</returns>
      bool     isUpdateVolume();

      /// <returns>Notes whether the trade is a cancel for a previous trade.</returns>
      bool     isCancel();

      /// <returns>Notes whether the trade is from an ECN book.</returns>
      bool     isFromBook(); 

      /// <returns>Trade price.</returns>
      double   getPrice();       

      /// <returns>Accumulated volume for the trading day.</returns>
      __int64  getVolume();

      /// <summary>
      /// Sets ID of appropriate request.
      /// </summary>
      /// <param name="requestId">ID of appropriate request.</param>
      void setRequestId (int requestId);

      /// <summary>Sets time of the event.</summary>
      /// <param name="timestamp">Time when the event occurred.</param>
      void setTimestamp (int timestamp);

      /// <summary>
      /// Sets trade price.
      /// </summary>
      /// <param name="value">Trade price.</param>
      void setPrice (double value);

      /// <summary>
      /// Sets number of shares traded in this transaction.
      /// </summary>
      /// <param name="value">Number of shares traded in this transaction.</param>
      void setSize (int value);

      /// <summary>
      /// Sets accumulated volume for the trading day.
      /// </summary>
      /// <param name="value">Accumulated volume for the trading day.</param>
      void setVolume (__int64 value);

      /// <summary>
      /// Sets sequence number of trade.
      /// </summary>
      /// <param name="sequenceNumber">Sequence number of trade.</param>
      void setSequenceNumber (int sequenceNumber);

      /// <summary>
      /// Sets indicator.
      /// </summary>
      /// <param name="indicator">Indicator.</param>
      void setIndicator (char indicator);

      /// <summary>
      /// Sets tick indicator: D - Down, U - Up, N - Not provided.
      /// </summary>
      /// <param name="tickIndicator">Tick indicator: D - Down, U - Up, N - Not provided.</param>
      void setTickIndicator (char tickIndicator);

      /// <summary>
      /// Sets flags.
      /// </summary>
      /// <param name="flags">Flags.</param>
      void setFlags (char flags);

      /// <summary>
      /// Sets symbol code.
      /// </summary>
      /// <param name="symbol">Symbol code.</param>
      void setSymbol (char *symbol);

      /// <summary>
      /// Sets exchange code.
      /// </summary>
      /// <param name="exchange">Exchange code.</param>
	  void setExchange (char *exchange);
	  
    private:
      OTTrade (const OTTrade &);
      OTTrade & operator = (const OTTrade &);

      int      requestId;
      int      timestamp;
      int      size;
      int      sequenceNumber;
      char     indicator;
      char     tickIndicator;
      char     flags;
      char    *symbol;
      char    exchange[3];
      double   price;
      __int64  volume;      
  };


#endif
